//#include <UTFT.h>
#include <UTFTGLUE.h>              //use GLUE class and constructor
#include "SystemStats.h" 
#include <avr/pgmspace.h>


extern uint8_t SmallFont[];
extern uint8_t SixteenSegment40x60[]; 
extern uint8_t SixteenSegment16x24[]; 

//UTFT myGLCD(HX8357B,38,39,40,41);
UTFTGLUE myGLCD(0,A2,A1,A3,A4,A0); //all dummy args


bool firstPass = false;
int i = 0;

const byte SEPERATOR = 124;
 long SerialRate = 115200;//115200;
 int graphSize = 460;
 int bytesRead = 0;
 int memoryGraphPos = 0;
 int processorGraphPos = 0;
 byte empty[0];
 byte processorGraph[460];
 byte memoryGraph[460];
 byte txGraph[460];
 byte rxGraph[460];
 String oldTime = "";
 String oldDate = "";
 String oldUptime = "";
 
void setup()
{
  Serial.begin(SerialRate);

  //delay(2000);

// Setup the LCD
  myGLCD.InitLCD();
  myGLCD.setFont(SmallFont);


  for(i=0;i<sizeof(processorGraph);i++)
    processorGraph[i] = 101;

  for(i=0;i<sizeof(memoryGraph);i++)
    memoryGraph[i] = 101;

}

//green 255, 0, 255
//red 0, 255, 255
//orange 0, 180, 255
//

//consts
 const int LABEL_HEIGHT = 10;
 const int WIDTH = 480;
 const int HEIGHT = 320;
 const int MARGIN = 10;
 const int CENTER_SPACE = 10;
 const int LABEL_SPACE = 10;
 int HEIGHT_OF_CORE_BARS = 0;
 int WIDTH_OF_CORE_BARS = 0;
 int NUM_CORES = 0;






//the colors for this screen are CMY, this method allows easy color selection
void mySetForeColor(AColor color) {
  switch(color)
  {
    case red  : myGLCD.setColor(0, 255, 255);   break;
    case green: myGLCD.setColor(255, 0, 255);   break;
    case blue : myGLCD.setColor(255, 0, 0);  break;
    case orange : myGLCD.setColor(0, 180, 255);  break;
    case black : myGLCD.setColor(255, 255, 255);  break;
  }
}

void mySetBackColor(AColor color) {
  switch(color)
  {
    case red  : myGLCD.setBackColor(0, 255, 255);   break;
    case green: myGLCD.setBackColor(255, 0, 255);   break;
    case blue : myGLCD.setBackColor(255, 0, 0);  break;
    case orange : myGLCD.setBackColor(0, 180, 255);  break;
    case black : myGLCD.setBackColor(255, 255, 255);  break;
  }
}

void drawText(int x, int y, String st, AColor color, int size) {
      if(size==40)
        myGLCD.setFont(SixteenSegment40x60);
      else if(size==20)
        myGLCD.setFont(SixteenSegment16x24);
      else
        myGLCD.setFont(SmallFont);

      mySetBackColor(black);
      mySetForeColor(color);
      myGLCD.print(st, x, y);
}

void drawPercentBar(int x, int y, int width, int height, int percent) {
      
      double pixelsPerPercent = width/100.0;

      //clear the old data
      mySetBackColor(black);
      mySetForeColor(black);
      myGLCD.fillRect(x+(percent*pixelsPerPercent), y, x+width, y+height);//clear just the part of the bar graph no longer relevant


      if (percent >= 0) 
      {
          mySetBackColor(green);
          mySetForeColor(green);
          myGLCD.fillRect(x, y, x + ((int)((percent>=50 ? 50 : percent) * pixelsPerPercent)), y + height);

        if (percent > 50) {
          mySetBackColor(orange);
          mySetForeColor(orange);
          myGLCD.fillRect(
            x+(int)(pixelsPerPercent * 50), 
            y, 
            (x+(int)(pixelsPerPercent * 50)) + (int)((percent >= 75 ? 75 - 50 : percent - 50) * pixelsPerPercent), 
            y + height
            );
        }

        if (percent > 75) {
          mySetBackColor(red);
          mySetForeColor(red);
          myGLCD.fillRect(
            x+(int)(pixelsPerPercent * 75), 
            y, 
            (x+(int)(pixelsPerPercent * 75)) + (int)((percent - 75) * pixelsPerPercent), 
            y + height
            );
        }
      }
}


void drawProgressGraphItem(int x, int y, int width, int height, byte dataSet[], AColor dsColor, String dsCaption, int dsNewValue, int rectangleBoxHeight, double pixelsPerPercent, int currentXPos)
{
      int yPos = y + LABEL_HEIGHT + 5;
      int ypos2 = yPos + rectangleBoxHeight;

        int prevBar = currentXPos - 1;

        if(prevBar < 0)
         prevBar = graphSize - 1;
         
        mySetForeColor(black);
        myGLCD.drawLine (x + prevBar, yPos, x + prevBar, ypos2);
        
        
        mySetForeColor(red);
        myGLCD.drawLine (x + currentXPos, yPos, x + currentXPos, ypos2);    Serial.begin(SerialRate);


      
        int yPosProgress2 = (int)(ypos2 - (dataSet [0] * pixelsPerPercent));
        int xPosProgress2 = x;
        int yPosProgress1 = 0;
        int xPosProgress1 = 0;

        dataSet[currentXPos] = (byte)dsNewValue;

        
        mySetForeColor(dsColor);

        //render the new graph
        yPosProgress2 = (int)(ypos2 -(dataSet [0] * pixelsPerPercent));
        xPosProgress2 = x;
        yPosProgress1 = 0;
        xPosProgress1 = 0;

        for (i = 0; i< graphSize; i++) {
          if (dataSet [i] <= 100) {
            yPosProgress1 = yPosProgress2;
            xPosProgress1 = xPosProgress2;
            yPosProgress2 = (int)(ypos2 - (dataSet [i] * pixelsPerPercent));
            xPosProgress2 = x + i;
            myGLCD.drawLine (xPosProgress1, yPosProgress1, xPosProgress2, yPosProgress2);
          }
        }
}

void drawProgressGraph(int x, int y, int width, int height, String label, byte dataSet1[], AColor ds1Color, String ds1Caption, int ds1NewValue,byte dataSet2[], AColor ds2Color, String ds2Caption, int ds2NewValue, int currentXPos)
{
   drawText(x, y, label, blue, 10);
   int rectangleBoxHeight = height - 10;
   int yPos = y + LABEL_HEIGHT + 5;

   mySetForeColor(blue);

   double pixelsPerPercent = rectangleBoxHeight / 100.0;

   drawProgressGraphItem (x, y, width, height, dataSet1, ds1Color, ds1Caption, ds1NewValue, rectangleBoxHeight, pixelsPerPercent, currentXPos);
}

void renderScreen(byte paramaters[])
{
  WIDTH_OF_CORE_BARS = (int)((WIDTH - (MARGIN*2) + CENTER_SPACE) / 2.0);

  NUM_CORES = 0;

  int pipeCnt = 0;
  int PROCESSOR_TOTAL_VAL = 0;
  int MEMORY_VAL = 0;
  int NET_RX_VAL = 0;
  int NET_TX_VAL = 0;
  String TIME = "";
  String DATE = "";
  String UPTIME = "";
  String CPU_TEMP = "";
  String FREE_DISK = "";

  for (int i = 0; i<bytesRead ; i++ ) {
    if (paramaters[i]==124)
    {
      pipeCnt++;
    }
    else if(pipeCnt==0)
      NUM_CORES++;
    else if(pipeCnt==1)
      PROCESSOR_TOTAL_VAL = paramaters [i];
    else if(pipeCnt==2)
      MEMORY_VAL =  paramaters [i];
    else if(pipeCnt==3)
      TIME = TIME + (char)paramaters [i];
    else if(pipeCnt==4)
      DATE = DATE + (char)paramaters [i];
    else if(pipeCnt==5)
      UPTIME = UPTIME + (char)paramaters [i];
  }

  double HH = ((HEIGHT - (MARGIN * 2) + CENTER_SPACE) / 2.0)-LABEL_HEIGHT-LABEL_SPACE;
  HEIGHT_OF_CORE_BARS = (int)(HH/NUM_CORES);

  byte corePercent[NUM_CORES];

  for (int i = 0; i< NUM_CORES; i++ ) {
    corePercent [i] = paramaters [i];
  }

  int xPos = MARGIN;
  int yPos = MARGIN;

  drawText (xPos, yPos, "Cores", blue,10);

  yPos += (LABEL_HEIGHT + LABEL_SPACE);

  for (int i = 0; i< NUM_CORES; i++ ) {
    corePercent [i] = paramaters [i];
    drawPercentBar (xPos, yPos, WIDTH_OF_CORE_BARS, HEIGHT_OF_CORE_BARS, paramaters [i]);
    yPos += HEIGHT_OF_CORE_BARS;
  }

  yPos += CENTER_SPACE;

  int graphHeight = (int)((HEIGHT - yPos - (LABEL_SPACE * 2) - MARGIN) / 2.0);//there are three graphs

  if(processorGraphPos>=graphSize)
    processorGraphPos = 0;
     
  drawProgressGraph (xPos, yPos, WIDTH - (MARGIN * 3), graphHeight, "Processor", processorGraph, blue, "",PROCESSOR_TOTAL_VAL, empty, red, "",0, processorGraphPos);

   processorGraphPos++;
   
  yPos += graphHeight + LABEL_HEIGHT;

  if(memoryGraphPos>=graphSize)
    memoryGraphPos = 0;

  drawProgressGraph (xPos, yPos, WIDTH - (MARGIN * 3), graphHeight, "Memory", memoryGraph, blue, "",MEMORY_VAL, empty, red, "",0, memoryGraphPos);

  memoryGraphPos++;
  //yPos += graphHeight + LABEL_HEIGHT;

  //drawProgressGraph (xPos, yPos, WIDTH - (MARGIN * 3), graphHeight, "Network", txGraph, blue, "",NET_RX_VAL, rxGraph, green, "",NET_TX_VAL);


//Move to the right Side of the Screen

  xPos = (int)((WIDTH/2.0) + (CENTER_SPACE/2.0));
  yPos = MARGIN;

  if(oldTime!=TIME)
  {
    drawText(xPos,yPos,TIME,blue,40);
    oldTime=TIME;
  }

  yPos+=75;
  if(oldDate!=DATE)
  {
    drawText(xPos,yPos,DATE,blue,20);
    oldDate=DATE;
  }
  yPos+=55;

  if(oldUptime!=TIME)
  {
    drawText(xPos,yPos,UPTIME,blue,20);
    oldUptime=UPTIME;
  }
 }

bool isReading = false;

int numRead = 0;

byte paramaters[100];
 
void loop() {

  if(firstPass==false)
  {
    myGLCD.clrScr();
    myGLCD.setColor(255, 255, 255);
    myGLCD.fillRect(0, 0, 479, 319);
    
        drawText (10, 10, "BEGIN", blue,10);
    firstPass=true;
  }
  

  while (!Serial.available()) {
     //drawText (10, 20, "WAITING FOR DATA", blue,10);

    } // wait for data to arrive  


  isReading = false;

 
  
  while (Serial.available()) // this will be skipped if no data present, leading to
                             // the code sitting in the delay function below
  {
    //drawText (10, 30, "DATA AVAILABLE", blue,10);
 
    delay(10);  //delay to allow buffer to fill 
    if (Serial.available() >0)
    {
      char c = Serial.read();  //gets one byte from serial buffer

      //drawText (10, 40, "R:" + c, blue,10);
      
      if(c=='}')
      {
       //paramaters = "";
       isReading = true;
       bytesRead = 0;
      }
      else if(isReading&&c=='~')
      {
        numRead++;
        if(bytesRead>0)
        {
          renderScreen(paramaters);
          //drawText (10, 70, "123:"+String(bytesRead)+"-"+String(numRead), blue,40);
        bytesRead = 0;
        isReading = false;
        Serial.write(1);
        }
       break;
      }
      else
      {
        paramaters[bytesRead] = (byte)c;
        bytesRead++;
      }
    }
  }


}
