using System;
using System.Collections.Generic;
using System.Diagnostics;
using System.IO.Ports;
using System.Linq;
using System.Management;
using System.Runtime.InteropServices.ComTypes;
using System.Text;
using System.Threading;
using System.Threading.Tasks;

namespace SystemStatusArd
{
    class Program
    {
        static void Main(string[] args)
        {
            StatsMonitor sm = new StatsMonitor(args[0]);
            sm.Begin();
        }
    }

    public class StatsMonitor
    {
        private static SerialPort port;
        private static List<System.Diagnostics.PerformanceCounter> corePC = new List<System.Diagnostics.PerformanceCounter>();
        private static PerformanceCounter totalCPUPC = new PerformanceCounter("Processor", "% Processor Time", "_Total");
        private static PerformanceCounter upTimePC = new PerformanceCounter("System", "System Up Time");

        public StatsMonitor(String comPort)
        {
            SetupCounters();
            port = new SerialPort(comPort, 115200);
            port.Open();
            Thread.Sleep(3000);
        }

        private static bool running = false;
        static public void _Tick(Object stateInfo)
        {


            if (running == false)
            {
                running = true;
                string cores = "";

                foreach (System.Diagnostics.PerformanceCounter pc in corePC)
                    cores += (char) Convert.ToInt32(Math.Floor(pc.NextValue()));

                string totalCPU = "" + (char) Convert.ToInt32(Math.Floor(totalCPUPC.NextValue()));

                upTimePC.NextValue(); //Call this an extra time before reading its value

                upTimePC.NextValue();
                TimeSpan ts = TimeSpan.FromSeconds(upTimePC.NextValue());

                string uptimeStr = ts.Days + "D " + ts.Hours + "H " + ts.Minutes + "M   ";


                string outPut = "}" + cores + "|" + totalCPU + "|" + (char)getFreeMemory() + "|" + DateTime.Now.ToString("HH:mm") +
                                "|" + DateTime.Now.ToString("yyyy-dd-MMM").ToUpper() + "|" + uptimeStr + "|" + "~";
                byte[] bytes = Encoding.Default.GetBytes(outPut);

                string outString = Encoding.UTF8.GetString(bytes);
                port.Write(outString);
                running = false;
                port.ReadChar();
            }

        }

        private static int getFreeMemory()
        {
            var wmiObject = new ManagementObjectSearcher("select * from Win32_OperatingSystem");

            var memoryValues = wmiObject.Get().Cast<ManagementObject>().
                Select(mo => new
                {
                    FreePhysicalMemory = Double.Parse(mo["FreePhysicalMemory"].ToString()),
                    TotalVisibleMemorySize = Double.Parse(mo["TotalVisibleMemorySize"].ToString())
                }).FirstOrDefault();

            if (memoryValues != null)
            {
                var percent = (int)(((memoryValues.TotalVisibleMemorySize - memoryValues.FreePhysicalMemory) / memoryValues.TotalVisibleMemorySize) * 100);
                return percent;
            }

            return 0;
        }

        private static void SetupCounters()
        {
            int procCount = System.Environment.ProcessorCount;
            for (int i = 0; i < procCount; i++)
            {
                System.Diagnostics.PerformanceCounter pc = new System.Diagnostics.PerformanceCounter("Processor", "% Processor Time", i.ToString());
                corePC.Add(pc);
            }
        }

        public void Begin()
        {
            TimerCallback callback = new TimerCallback(_Tick);

            Timer stateTimer = new Timer(callback, null, 0, 1000);

            // loop here forever
            for (;;)
            {
                Thread.Sleep(500);
            }
        }
    }
}

